from typing import Callable, Any
from datetime import datetime
from functools import partial

from emptylog.protocol import LoggerProtocol


class PrintingLogger(LoggerProtocol):
    def __init__(self, printing_callback: Callable[[Any], Any] = partial(print, end=''), separator: str = '|') -> None:
        self.callback = printing_callback
        self.separator = separator
        self.template = '{time} {separator} {level} {separator} {message}\n'

    def debug(self, message: str, *args: Any, **kwargs: Any) -> None:
        self.callback(self.create_line('DEBUG', message))

    def info(self, message: str, *args: Any, **kwargs: Any) -> None:
        self.callback(self.create_line('INFO', message))

    def warning(self, message: str, *args: Any, **kwargs: Any) -> None:
        self.callback(self.create_line('WARNING', message))

    def error(self, message: str, *args: Any, **kwargs: Any) -> None:
        self.callback(self.create_line('ERROR', message))

    def exception(self, message: str, *args: Any, **kwargs: Any) -> None:
        self.callback(self.create_line('EXCEPTION', message))

    def critical(self, message: str, *args: Any, **kwargs: Any) -> None:
        self.callback(self.create_line('CRITICAL', message))

    def create_line(self, level: str, message: str) -> str:
        return self.template.format(time=datetime.now(), level=level.ljust(9), message=message, separator=self.separator)



import logging

from emptylog import EmptyLogger


def test_search_callable_attributes():
    attribute_names = [
        'debug',
        'info',
        'warning',
        'error',
        'exception',
        'critical',
    ]

    empty_logger = EmptyLogger()
    real_logger = logging.getLogger('kek')

    for name in attribute_names:
        for logger in [empty_logger, real_logger]:
            method = getattr(logger, name)

            assert callable(method)

            method('kek')
            method('kek %s', 'lol')
            method('kek %s', 'lol', extra={'lol': 'kek'})



from typing import Any

from emptylog.protocol import LoggerProtocol
from emptylog.call_data import LoggerCallData
from emptylog.accumulated_data import LoggerAccumulatedData


class MemoryLogger(LoggerProtocol):
    def __init__(self) -> None:
        self.data = LoggerAccumulatedData()

    def debug(self, message: str, *args: Any, **kwargs: Any) -> None: self.data.debug.append(LoggerCallData(message, args, kwargs))
    def info(self, message: str, *args: Any, **kwargs: Any) -> None: self.data.info.append(LoggerCallData(message, args, kwargs))
    def warning(self, message: str, *args: Any, **kwargs: Any) -> None: self.data.warning.append(LoggerCallData(message, args, kwargs))
    def error(self, message: str, *args: Any, **kwargs: Any) -> None: self.data.error.append(LoggerCallData(message, args, kwargs))
    def exception(self, message: str, *args: Any, **kwargs: Any) -> None: self.data.exception.append(LoggerCallData(message, args, kwargs))
    def critical(self, message: str, *args: Any, **kwargs: Any) -> None: self.data.critical.append(LoggerCallData(message, args, kwargs))



from emptylog.call_data import LoggerCallData


def test_create_call_data_and_check_the_data():
    data = LoggerCallData('kek', (), {})

    assert data.message == 'kek'
    assert data.args == ()
    assert data.kwargs == {}



from typing import Any

try:
    from typing import Protocol, runtime_checkable
except ImportError:  # pragma: no cover
    from typing_extensions import Protocol, runtime_checkable  # type: ignore[assignment]


@runtime_checkable
class LoggerProtocol(Protocol):
    def debug(self, message: str, *args: Any, **kwargs: Any) -> None: return None
    def info(self, message: str, *args: Any, **kwargs: Any) -> None: return None
    def warning(self, message: str, *args: Any, **kwargs: Any) -> None: return None
    def error(self, message: str, *args: Any, **kwargs: Any) -> None: return None
    def exception(self, message: str, *args: Any, **kwargs: Any) -> None: return None
    def critical(self, message: str, *args: Any, **kwargs: Any) -> None: return None



import logging

from emptylog import LoggerProtocol, EmptyLogger


def test_positive_examples_of_runtime_check():
    assert isinstance(logging, LoggerProtocol)
    assert isinstance(logging.getLogger('kek'), LoggerProtocol)
    assert isinstance(logging.getLogger('kek'), LoggerProtocol)
    assert isinstance(EmptyLogger(), LoggerProtocol)


def test_negative_examples_of_runtime_check():
    assert not isinstance(1, LoggerProtocol)
    assert not isinstance('logging', LoggerProtocol)
